@@ -5,6 +5,8 @@ from rest_framework.settings import APISettings
 DEFAULTS = {
     "DEFAULT_PAGE_SIZE": 25,
     "DEFAULT_PERMISSION_CLASSES": ("djangochannelsrestframework.permissions.AllowAny",),
+    "DEFAULT_PAGINATION_CLASS": None,
+    "PAGE_SIZE": None,
 }
 IMPORT_STRINGS = ("DEFAULT_PERMISSION_CLASSES",)
 
